;; Copyright 2009, 2010, 2014, 2015 Kevin Ryde

;; This file is part of pcf2bdf-el.
;;
;; pcf2bdf-el is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the
;; Free Software Foundation; either version 3, or (at your option) any later
;; version.
;;
;; pcf2bdf-el is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
;; Public License for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with pcf2bdf-el; see the file COPYING.  Failing that, go to
;; <http://www.gnu.org/licenses/>.

(add-to-list 'load-path (expand-file-name ".."))

(defconst my-pcf2bdf-available-p
  ;; pcf2bdf version 1.04 doesn't have a --version.  It exits with status 1
  ;; when it prints the usage message.
  (condition-case nil
      (member (call-process "pcf2bdf"
                            nil  ;; stdin
                            nil  ;; stdout
                            nil  ;; display
                            "--version")
              '(0 1))
    (error nil))
  "Non-nil if the \"pcf2bdf\" program is available.")

(defconst my-bdftopcf-available-p
  ;; bdftopcf doesn't have a --version, as of Xorg 7.5.  It exits with
  ;; status 1 when it prints the usage message.
  (condition-case nil
      (member (call-process "bdftopcf"
                            nil  ;; stdin
                            nil  ;; stdout
                            nil  ;; display
                            "--version")
              '(0 1))
    (error nil))
  "Non-nil if the \"bdftopcf\" program is available.")

(defun my-run-test ()
  (let ((count 0))

    ;;-------------------------------------------------------------------------
    ;; load

    (require 'pcf2bdf)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    (require 'compile)

    ;; no leftovers in emacs21 byte compiling defadvice
    (when (and (boundp 'byte-compile-unresolved-functions)
               byte-compile-unresolved-functions)
      (error "byte-compile-unresolved-functions after load 'compile: %S"
             byte-compile-unresolved-functions))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; compile eval-after-load working

    (or (and (boundp 'compilation-error-regexp-systems-list)
             ;; xemacs21
             (assq 'pcf2bdf--bdftopcf compilation-error-regexp-alist-alist))

        (and (boundp 'compilation-error-regexp-alist-alist)
             ;; emacs22
             (memq 'pcf2bdf--bdftopcf compilation-error-regexp-alist))

        ;; emacs21
        (let (found)
          (mapcar (lambda (elem)
                    (setq elem (car-safe elem))
                    (and (stringp elem)
                         (string-match "BDF" elem)
                         (setq found t)))
                  compilation-error-regexp-alist)
          found)

        (error "pcf2bdf compile regexp not applied"))
    (setq count (1+ count))


    ;;-------------------------------------------------------------------------
    ;; pcf2bdf--with-errorfile

    (if (not (fboundp 'pcf2bdf--with-errorfile))
        (message "skip, no pcf2bdf--with-errorfile when byte compiled")

      (pcf2bdf--with-errorfile
       (unless (boundp 'errorfile)
         (error "pcf2bdf--with-errorfile doesn't bind `errorfile'"))
       (message "t-pcf2bdf.el: errorfile is %S" errorfile))
      (setq count (1+ count)))


    ;;-------------------------------------------------------------------------
    (if (not my-pcf2bdf-available-p)
        (message "  skip, pcf2bdf program not available")

      ;; decode working from a non-existent directory
      (let ((buffer (get-buffer-create "*t-pcf2bdf test*")))
        (switch-to-buffer buffer)
        (if (fboundp 'set-buffer-multibyte) ;; not in xemacs21
            (set-buffer-multibyte nil))
        (insert-file-contents-literally "t-pcf2bdf.pcf")
        (setq default-directory "/no/such/dir/exists/")
        (pcf2bdf)
        (kill-buffer buffer)
        (setq count (1+ count)))

      ;;-----------------------------------------------------------------------
      ;; saving

      (if (not my-bdftopcf-available-p)
          (message "  skip, bdftopcf program not available")

        (let ((tempfile "t-pcf2bdf--temp.pcf"))
          (copy-file "t-pcf2bdf.pcf" tempfile)
          (unwind-protect
              (progn
                (find-file "t-pcf2bdf--temp.pcf")
                (goto-char (point-min))
                (when (re-search-forward "SIZE -1 " nil t)
                  (goto-char (- (point) 3))
                  (delete-region (point) (+ (point) 2))
                  (insert "64"))
                (set-buffer-modified-p t)
                (save-buffer)

                (kill-buffer nil)
                (find-file "t-pcf2bdf--temp.pcf")
                (kill-buffer nil)

                (setq count (1+ count)))

            (when (file-exists-p tempfile)
              (condition-case nil
		  (delete-file (file-newest-backup tempfile)) (error nil)))
            (condition-case nil
                (delete-file tempfile) (error nil))))
        )

      )
    ;;-------------------------------------------------------------------------
    ;; unload-feature

    (add-to-list 'load-path (expand-file-name "."))
    (require 'my-unload)

    (if (not my-unload-feature-function-p)
        (message "skip, no unload-feature function")

      (let ((feature 'pcf2bdf))
        (unless (my-advice-by-name feature)
          (error "oops, didn't find any advice"))

        (unload-feature feature t)

        (let ((lst (my-advice-by-name feature)))
          (when lst
            (error "unload-feature leftover advice: %S" lst)))
        (setq count (1+ count))))

    ;;-----------------------------------------------------------------------
    ;; unload-feature -- with advice unloaded first

    (let ((feature 'pcf2bdf))
      (if (featurep feature)
          (unload-feature feature t))
      (require feature)
      ;; if 'advice can be unloaded (no dependency expressed on it), then
      ;; see FEATURE unloads successfully too
      (when (condition-case nil
                (progn (unload-feature 'advice) t)
              (error nil))
        (unload-feature feature t)))
    (setq count (1+ count))

    ;;-------------------------------------------------------------------------

    (message "  %d tests" count)
    count))
